/**
 * @file
 * VuoList_VuoVideoInputDevice C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoVideoInputDevice_H
#define VuoList_VuoVideoInputDevice_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoVideoInputDevice VuoList_VuoVideoInputDevice
 * A list of @ref VuoVideoInputDevice elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoVideoInputDevice_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoVideoInputDevice elements.
 */
typedef const struct VuoList_VuoVideoInputDevice_struct { void *l; } * VuoList_VuoVideoInputDevice;
/// @}
#endif

/**
 * Creates a new list of @ref VuoVideoInputDevice elements.
 */
VuoList_VuoVideoInputDevice VuoListCreate_VuoVideoInputDevice(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoVideoInputDevice VuoListCopy_VuoVideoInputDevice(const VuoList_VuoVideoInputDevice list);

/**
 * Returns the @ref VuoVideoInputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoVideoInputDevice VuoListGetValue_VuoVideoInputDevice(const VuoList_VuoVideoInputDevice list, const unsigned long index);

/**
 * Changes the @ref VuoVideoInputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoVideoInputDevice(const VuoList_VuoVideoInputDevice list, const VuoVideoInputDevice value, const unsigned long index);

/**
 * Inserts the `VuoVideoInputDevice` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoVideoInputDevice(const VuoList_VuoVideoInputDevice list, const VuoVideoInputDevice value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoVideoInputDevice(VuoList_VuoVideoInputDevice list, const VuoVideoInputDevice value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoVideoInputDevice(VuoList_VuoVideoInputDevice list, const VuoVideoInputDevice value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoVideoInputDevice(VuoList_VuoVideoInputDevice list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoVideoInputDevice(VuoList_VuoVideoInputDevice list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoVideoInputDevice(VuoList_VuoVideoInputDevice list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoVideoInputDevice(VuoList_VuoVideoInputDevice list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoVideoInputDevice(VuoList_VuoVideoInputDevice list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoVideoInputDevice(VuoList_VuoVideoInputDevice list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoVideoInputDevice(VuoList_VuoVideoInputDevice list);

/**
 * Removes the `VuoVideoInputDevice` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoVideoInputDevice(VuoList_VuoVideoInputDevice list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoVideoInputDevice(const VuoList_VuoVideoInputDevice list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoVideoInputDevice VuoList_VuoVideoInputDevice_valueFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoVideoInputDevice_jsonFromValue(const VuoList_VuoVideoInputDevice value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoVideoInputDevice_summaryFromValue(const VuoList_VuoVideoInputDevice value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoVideoInputDevice VuoList_VuoVideoInputDevice_valueFromString(const char *str);
char * VuoList_VuoVideoInputDevice_stringFromValue(const VuoList_VuoVideoInputDevice value);
/// @}

/**
 * @}
 */

#endif
